@@ -14,5 +14,5 @@ extern void ladder_base(fe *, const unsigned char *);
 }
 #endif
 
-#endif //ifndef ladder_base_H
+#endif /* ifndef ladder_base_H */
 
